@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -15,211 +15,13 @@ import copy
 import os
 import unittest
 
-import trac.tests.compat
 from trac.config import ConfigurationError
 from trac.db.api import DatabaseManager, get_column_names, \
-                        parse_connection_uri, with_transaction
+                        parse_connection_uri
 from trac.db_default import (schema as default_schema,
                              db_version as default_db_version)
 from trac.db.schema import Column, Table
-from trac.test import EnvironmentStub, Mock, get_dburi
-from trac.util.concurrency import ThreadLocal
-
-
-class Connection(object):
-
-    committed = False
-    rolledback = False
-
-    def commit(self):
-        self.committed = True
-
-    def rollback(self):
-        self.rolledback = True
-
-
-class Error(Exception):
-    pass
-
-
-def make_env(get_cnx):
-    from trac.core import ComponentManager
-    return Mock(ComponentManager, components={DatabaseManager:
-             Mock(get_connection=get_cnx,
-                  _transaction_local=ThreadLocal(wdb=None, rdb=None))})
-
-
-class WithTransactionTest(unittest.TestCase):
-
-    def test_successful_transaction(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        @with_transaction(env)
-        def do_transaction(db):
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(db.committed and not db.rolledback)
-
-    def test_failed_transaction(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        try:
-            @with_transaction(env)
-            def do_transaction(db):
-                self.assertTrue(not db.committed and not db.rolledback)
-                raise Error()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(not db.committed and db.rolledback)
-
-    def test_implicit_nesting_success(self):
-        env = make_env(Connection)
-        dbs = [None, None]
-        @with_transaction(env)
-        def level0(db):
-            dbs[0] = db
-            @with_transaction(env)
-            def level1(db):
-                dbs[1] = db
-                self.assertTrue(not db.committed and not db.rolledback)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(dbs[0].committed and not dbs[0].rolledback)
-
-    def test_implicit_nesting_failure(self):
-        env = make_env(Connection)
-        dbs = [None, None]
-        try:
-            @with_transaction(env)
-            def level0(db):
-                dbs[0] = db
-                try:
-                    @with_transaction(env)
-                    def level1(db):
-                        dbs[1] = db
-                        self.assertTrue(not db.committed and not db.rolledback)
-                        raise Error()
-                    self.fail()
-                except Error:
-                    self.assertTrue(not db.committed and not db.rolledback)
-                    raise
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and dbs[0].rolledback)
-
-    def test_explicit_success(self):
-        db = Connection()
-        env = make_env(lambda: None)
-        @with_transaction(env, db)
-        def do_transaction(idb):
-            self.assertTrue(idb is db)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(not db.committed and not db.rolledback)
-
-    def test_explicit_failure(self):
-        db = Connection()
-        env = make_env(lambda: None)
-        try:
-            @with_transaction(env, db)
-            def do_transaction(idb):
-                self.assertTrue(idb is db)
-                self.assertTrue(not db.committed and not db.rolledback)
-                raise Error()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(not db.committed and not db.rolledback)
-
-    def test_implicit_in_explicit_success(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        @with_transaction(env, db)
-        def level0(db):
-            dbs[0] = db
-            @with_transaction(env)
-            def level1(db):
-                dbs[1] = db
-                self.assertTrue(not db.committed and not db.rolledback)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and not dbs[0].rolledback)
-
-    def test_implicit_in_explicit_failure(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        try:
-            @with_transaction(env, db)
-            def level0(db):
-                dbs[0] = db
-                @with_transaction(env)
-                def level1(db):
-                    dbs[1] = db
-                    self.assertTrue(not db.committed and not db.rolledback)
-                    raise Error()
-                self.fail()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and not dbs[0].rolledback)
-
-    def test_explicit_in_implicit_success(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        @with_transaction(env)
-        def level0(db):
-            dbs[0] = db
-            @with_transaction(env, db)
-            def level1(db):
-                dbs[1] = db
-                self.assertTrue(not db.committed and not db.rolledback)
-            self.assertTrue(not db.committed and not db.rolledback)
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(dbs[0].committed and not dbs[0].rolledback)
-
-    def test_explicit_in_implicit_failure(self):
-        db = Connection()
-        env = make_env(lambda: db)
-        dbs = [None, None]
-        try:
-            @with_transaction(env)
-            def level0(db):
-                dbs[0] = db
-                @with_transaction(env, db)
-                def level1(db):
-                    dbs[1] = db
-                    self.assertTrue(not db.committed and not db.rolledback)
-                    raise Error()
-                self.fail()
-            self.fail()
-        except Error:
-            pass
-        self.assertTrue(dbs[0] is not None)
-        self.assertTrue(dbs[0] is dbs[1])
-        self.assertTrue(not dbs[0].committed and dbs[0].rolledback)
-
-    def test_invalid_nesting(self):
-        env = make_env(Connection)
-        try:
-            @with_transaction(env)
-            def level0(db):
-                @with_transaction(env, Connection())
-                def level1(db):
-                    raise Error()
-                raise Error()
-            raise Error()
-        except AssertionError:
-            pass
+from trac.test import EnvironmentStub, get_dburi
 
 
 class ParseConnectionStringTestCase(unittest.TestCase):
@@ -330,7 +132,7 @@ class StringsTestCase(unittest.TestCase):
             "SELECT value FROM " + quoted + " WHERE name='test-empty'"))
 
     def test_insert_markup(self):
-        from genshi.core import Markup
+        from trac.util.html import Markup
         with self.env.db_transaction as db:
             quoted = db.quote('system')
             db("INSERT INTO " + quoted + " (name,value) VALUES (%s,%s)",
@@ -469,6 +271,41 @@ class ConnectionTestCase(unittest.TestCase):
         self.assertEqual((1, 'author1', 'comment one'), data[0])
         self.assertEqual((2, 'author2', 'comment two'), data[1])
 
+    def test_rollback_transaction_on_exception(self):
+        """Transaction is rolled back when an exception occurs in the
+        transaction context manager.
+        """
+        insert_sql = "INSERT INTO blog (bid, author) VALUES (42, 'anonymous')"
+        try:
+            with self.env.db_transaction as db:
+                db(insert_sql)
+                db(insert_sql)
+        except self.env.db_exc.IntegrityError:
+            pass
+
+        for _, in self.env.db_query("""
+                SELECT author FROM blog WHERE bid=42
+                """):
+            self.fail("Transaction was not rolled back")
+
+    def test_rollback_nested_transaction_on_exception(self):
+        """Transaction is rolled back when an exception occurs in the
+        inner transaction context manager.
+        """
+        sql = "INSERT INTO blog (bid, author) VALUES (42, 'anonymous')"
+        try:
+            with self.env.db_transaction as db_outer:
+                db_outer(sql)
+                with self.env.db_transaction as db_inner:
+                    db_inner(sql)
+        except self.env.db_exc.IntegrityError:
+            pass
+
+        for _, in self.env.db_query("""
+                SELECT author FROM blog WHERE bid=42
+                """):
+            self.fail("Transaction was not rolled back")
+
     def test_get_last_id(self):
         q = "INSERT INTO report (author) VALUES ('anonymous')"
         with self.env.db_transaction as db:
@@ -543,6 +380,12 @@ class ConnectionTestCase(unittest.TestCase):
                 self.assertEqual(column_names,
                                  db.get_column_names(table.name))
 
+    def test_get_column_names_non_existent_table(self):
+        with self.assertRaises(self.env.db_exc.OperationalError) as cm:
+            self.dbm.get_column_names('blah')
+        self.assertIn(unicode(cm.exception), ('Table "blah" not found',
+                                              'Table `blah` not found'))
+
 
 class DatabaseManagerTestCase(unittest.TestCase):
 
@@ -584,6 +427,12 @@ class DatabaseManagerTestCase(unittest.TestCase):
         self.assertEqual(sorted(table.name for table in default_schema),
                          sorted(self.dbm.get_table_names()))
 
+    def test_has_table(self):
+        self.assertIs(True, self.dbm.has_table('system'))
+        self.assertIs(True, self.dbm.has_table('wiki'))
+        self.assertIs(False, self.dbm.has_table('trac'))
+        self.assertIs(False, self.dbm.has_table('blah.blah'))
+
     def test_set_default_database_version(self):
         """Set database version for the default entry named
         `database_version`.
@@ -608,6 +457,17 @@ class DatabaseManagerTestCase(unittest.TestCase):
         self.dbm.set_database_version(db_ver, name)
         self.assertEqual(db_ver, self.dbm.get_database_version(name))
 
+    def test_get_sequence_names(self):
+        sequence_names = []
+        if self.dbm.connection_uri.startswith('postgres'):
+            for table in default_schema:
+                for column in table.columns:
+                    if column.name == 'id' and column.auto_increment:
+                        sequence_names.append(table.name)
+            sequence_names.sort()
+
+        self.assertEqual(sequence_names, self.dbm.get_sequence_names())
+
 
 class ModifyTableTestCase(unittest.TestCase):
 
@@ -634,9 +494,14 @@ class ModifyTableTestCase(unittest.TestCase):
         self.new_schema = copy.deepcopy([self.schema[0], self.schema[2]])
         self.new_schema[0].remove_columns(('col2',))
         self.new_schema[1].columns.append(Column('col4'))
+        self.new_schema.append(
+            Table('table4')[
+                Column('col1'),
+            ]
+        )
 
     def tearDown(self):
-        self.dbm.drop_tables(self.schema)
+        self.dbm.drop_tables(['table1', 'table2', 'table3', 'table4'])
         self.env.reset_db()
 
     def _insert_data(self):
@@ -675,6 +540,12 @@ class ModifyTableTestCase(unittest.TestCase):
         self.assertEqual((10,), data[0])
         self.assertEqual((13,), data[1])
 
+    def test_drop_columns_non_existent_table(self):
+        with self.assertRaises(self.env.db_exc.OperationalError) as cm:
+            self.dbm.drop_columns('blah', ('col1',))
+        self.assertIn(unicode(cm.exception), ('Table "blah" not found',
+                                              'Table `blah` not found'))
+
     def test_upgrade_tables_have_new_schema(self):
         """The upgraded tables have the new schema."""
         self.dbm.upgrade_tables(self.new_schema)
@@ -723,7 +594,6 @@ def test_suite():
     suite.addTest(unittest.makeSuite(ParseConnectionStringTestCase))
     suite.addTest(unittest.makeSuite(StringsTestCase))
     suite.addTest(unittest.makeSuite(ConnectionTestCase))
-    suite.addTest(unittest.makeSuite(WithTransactionTest))
     suite.addTest(unittest.makeSuite(DatabaseManagerTestCase))
     suite.addTest(unittest.makeSuite(ModifyTableTestCase))
     return suite
