@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # Copyright (C) 2008 Stephen Hansen
@@ -219,7 +219,8 @@ class NotificationSystem(Component):
         This component is used by the notification system to send emails.
         Trac currently provides `SmtpEmailSender` for connecting to an SMTP
         server, and `SendmailEmailSender` for running a `sendmail`-compatible
-        executable. (''since 0.12'')""")
+        executable.
+        """)
 
     smtp_enabled = BoolOption('notification', 'smtp_enabled', 'false',
         """Enable email notification.""")
