@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2020 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -15,20 +15,16 @@
 working with a Trac environment to make test cases more succinct.
 """
 
+import io
 import re
 
-from genshi.builder import tag
 from trac.tests.functional import internal_error
 from trac.tests.functional.better_twill import tc, b
 from trac.tests.contentgen import random_page, random_sentence, random_word, \
                                   random_unique_camel
+from trac.util.html import tag
 from trac.util.text import to_utf8, unicode_quote
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
-
 
 class FunctionalTester(object):
     """Provides a library of higher-level operations for interacting with a
@@ -263,7 +259,7 @@ class FunctionalTester(object):
         if content is None:
             content = random_page()
         self.go_to_wiki(name)
-        tc.find("The page %s does not exist." % tag.strong(name))
+        tc.find("The page[ \n]+%s[ \n]+does not exist." % tag.strong(name))
 
         self.edit_wiki_page(name, content, comment)
 
@@ -467,7 +463,7 @@ class FunctionalTester(object):
 
         tc.submit('attachfilebutton', 'attachfile')
         tc.url(self.url + r'/attachment/%s/%s/\?action=new$' % (realm, name))
-        fp = StringIO(data)
+        fp = io.BytesIO(data)
         tc.formfile('attachment', 'attachment', filename,
                     content_type=content_type, fp=fp)
         tc.formvalue('attachment', 'description', description)
