@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -100,7 +100,7 @@ if os.name == 'nt':
             st.wHour = st.wMinute = st.wSecond = st.wMilliseconds = 0
             ft = FILETIME()
             if SystemTimeToFileTime(ctypes.pointer(st), ctypes.pointer(ft)):
-                return ft.dwHighDateTime * 0x100000000L + ft.dwLowDateTime
+                return ft.dwHighDateTime * 0x100000000 + ft.dwLowDateTime
             else:
                 raise RuntimeError('[LastError SystemTimeToFileTime %d]' %
                                    GetLastError())
@@ -113,8 +113,8 @@ if os.name == 'nt':
             if not get_systime(ctypes.pointer(ft)):
                 raise RuntimeError('[LastError %s %d]' %
                                    (func_systime, GetLastError()))
-            ft = ft.dwHighDateTime * 0x100000000L + ft.dwLowDateTime
-            usec = (ft - ft_epoch) / 10L
+            ft = ft.dwHighDateTime * 0x100000000 + ft.dwLowDateTime
+            usec = (ft - ft_epoch) / 10
             return usec / 1000000.0
 
         def datetime_now(tz=None):
@@ -178,6 +178,12 @@ def to_datetime(t, tzinfo=None):
     raise TypeError('expecting datetime, int, long, float, or None; got %s' %
                     type(t))
 
+
+def truncate_datetime(dt):
+    """Truncate a datetime object to the start of the day."""
+    return dt.replace(hour=0, minute=0, second=0, microsecond=0)
+
+
 def to_timestamp(dt):
     """Return the corresponding POSIX timestamp"""
     if dt:
@@ -191,7 +197,7 @@ def to_utimestamp(dt):
     if not dt:
         return 0
     diff = dt - _epoc
-    return (diff.days * 86400000000L + diff.seconds * 1000000
+    return (diff.days * 86400000000 + diff.seconds * 1000000
             + diff.microseconds)
 
 def from_utimestamp(ts):
@@ -462,7 +468,7 @@ def get_timezone_list_jquery_ui(t=None):
         sign = '-' if offset < 0 else '+'
         return '%s%02d:%02d' % (sign, abs(offset // 60), offset % 60)
     t = to_datetime(t, utc)
-    offsets = set(utcoffset(tz, t) for tz in all_timezones)
+    offsets = {utcoffset(tz, t) for tz in all_timezones}
     return [{'value': offset, 'label': label(offset)}
             for offset in sorted(offsets)]
 
@@ -507,9 +513,10 @@ def get_period_names_jquery_ui(req):
         return {'am': [english_names['am']], 'pm': [english_names['pm']]}
     if babel and locale:
         names = get_period_names(locale=locale)
-        return dict((period, [names[period], english_names[period]]
-                             if period in names else [english_names[period]])
-                    for period in ('am', 'pm'))
+        return {period: [names[period], english_names[period]]
+                        if period in names
+                        else [english_names[period]]
+                for period in ('am', 'pm')}
     else:
         # retrieve names of am/pm from libc
         names = {}
@@ -619,7 +626,7 @@ def parse_date(text, tzinfo=None, locale=None, hint='date'):
             'date': get_date_format_hint,
             'relative': get_datetime_format_hint,
             'iso8601': lambda l: get_datetime_format_hint('iso8601'),
-        }.get(hint, lambda(l): hint)(locale)
+        }.get(hint, lambda l: hint)(locale)
         if locale == 'iso8601' and hint in ('date', 'datetime') or \
                 hint == 'iso8601':
             msg = _('"%(date)s" is an invalid date, or the date format '
@@ -674,7 +681,7 @@ def _i18n_parse_date_pattern(locale):
                     order.append((idx, key))
                     break
         order.sort()
-        orders.append(dict((key, idx) for idx, (_, key) in enumerate(order)))
+        orders.append({key: idx for idx, (_, key) in enumerate(order)})
 
     # always allow using English names regardless of locale
     month_names = dict(zip(('jan', 'feb', 'mar', 'apr', 'may', 'jun',
@@ -785,7 +792,7 @@ def _i18n_parse_date_0(text, order, regexp, period_names, month_names, tzinfo):
                 values[key], values['M'] = values['M'], value
             break
 
-    values = dict((key, int(value)) for key, value in values.iteritems())
+    values = {key: int(value) for key, value in values.iteritems()}
     values.setdefault('h', 0)
     values.setdefault('m', 0)
     values.setdefault('s', 0)
@@ -1145,15 +1152,15 @@ _timezones = [
     FixedOffset(600, 'GMT +10:00'),  FixedOffset(660, 'GMT +11:00'),
     FixedOffset(720, 'GMT +12:00'),  FixedOffset(780, 'GMT +13:00'),
     FixedOffset(840, 'GMT +14:00')]
-_tzmap = dict([(z.zone, z) for z in _timezones])
+_tzmap = {z.zone: z for z in _timezones}
 
 all_timezones = [z.zone for z in _timezones]
 
 try:
     import pytz
 
-    _tzoffsetmap = dict([(tz.utcoffset(None), tz) for tz in _timezones
-                         if tz.zone != 'UTC'])
+    _tzoffsetmap = {tz.utcoffset(None): tz
+                    for tz in _timezones if tz.zone != 'UTC'}
 
     def timezone(tzname):
         """Fetch timezone instance by name or raise `KeyError`"""
