@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -11,22 +11,18 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at https://trac.edgewall.org/log/.
 
+import io
 import os.path
-import sys
-import tempfile
 import textwrap
 import unittest
-from StringIO import StringIO
 
-import trac.tests.compat
-from genshi.builder import tag
 from trac import perm
 from trac.core import TracError
-from trac.test import EnvironmentStub, Mock, MockPerm, rmtree
+from trac.test import EnvironmentStub, MockPerm, mkdtemp, rmtree
 from trac.util import create_file
 from trac.util.datefmt import utc
-from trac.util.text import shorten_line
-from trac.web.api import HTTPBadRequest, HTTPInternalError, Request, \
+from trac.util.html import tag
+from trac.web.api import HTTPBadRequest, HTTPInternalServerError, Request, \
                          RequestDone, parse_arg_list
 from trac.web.main import FakeSession
 from tracopt.perm.authz_policy import AuthzPolicy
@@ -37,7 +33,7 @@ class RequestHandlerPermissionsTestCaseBase(unittest.TestCase):
     authz_policy = None
 
     def setUp(self, module_class):
-        self.path = tempfile.mkdtemp(prefix='trac-')
+        self.path = mkdtemp()
         if self.authz_policy is not None:
             self.authz_file = os.path.join(self.path, 'authz_policy.conf')
             create_file(self.authz_file, self.authz_policy)
@@ -69,36 +65,100 @@ class RequestHandlerPermissionsTestCaseBase(unittest.TestCase):
 def _make_environ(scheme='http', server_name='example.org',
                   server_port=80, method='GET', script_name='/trac',
                   **kwargs):
-    environ = {'wsgi.url_scheme': scheme, 'wsgi.input': StringIO(''),
+    environ = {'wsgi.url_scheme': scheme, 'wsgi.input': io.BytesIO(),
                'REQUEST_METHOD': method, 'SERVER_NAME': server_name,
                'SERVER_PORT': server_port, 'SCRIPT_NAME': script_name}
     environ.update(kwargs)
     return environ
 
 
-def _make_req(environ, start_response, args={}, arg_list=(), authname='admin',
-              form_token='A' * 40, chrome={'links': {}, 'scripts': []},
-              perm=MockPerm(), tz=utc, locale=None, **kwargs):
-    req = Request(environ, start_response)
-    req.args = args
-    req.arg_list = arg_list
+def _make_req(environ, authname='admin', chrome=None, form_token='A' * 40,
+              locale=None, perm=MockPerm(), tz=utc, use_xsendfile=False,
+              xsendfile_header='X-Sendfile'):
+    if chrome is None:
+        chrome = {
+            'links': {},
+            'scripts': [],
+            'theme': 'theme.html',
+            'logo': '',
+            'nav': ''
+        }
+
+    class RequestWithSentAttrs(Request):
+        """Subclass of `Request` with "sent" attributes."""
+
+        def __init__(self, environ):
+            self.status_sent = []
+            self.headers_sent = {}
+            self._response_sent = io.BytesIO()
+
+            def write(data):
+                self._response_sent.write(data)
+
+            def start_response(status, headers, exc_info=None):
+                self.status_sent.append(status)
+                self.headers_sent.update(dict(headers))
+                return write
+
+            super(RequestWithSentAttrs, self).__init__(environ, start_response)
+
+        @property
+        def response_sent(self):
+            return self._response_sent.getvalue()
+
+    req = RequestWithSentAttrs(environ)
+    # Setup default callbacks.
     req.authname = authname
-    req.form_token = form_token
     req.chrome = chrome
+    req.form_token = form_token
+    req.locale = locale
     req.perm = perm
     req.session = FakeSession()
     req.tz = tz
-    req.locale = locale
-    for name, value in kwargs.iteritems():
-        setattr(req, name, value)
+    req.use_xsendfile = use_xsendfile
+    req.xsendfile_header = xsendfile_header
     return req
 
 
 class RequestTestCase(unittest.TestCase):
 
+    def test_repr_with_path(self):
+        environ = _make_environ(PATH_INFO='/path')
+        req = Request(environ, None)
+        self.assertEqual(repr(req), """<Request "GET '/path'">""")
+
+    def test_repr_with_path_and_query_string(self):
+        environ = _make_environ(QUERY_STRING='A=B', PATH_INFO='/path')
+        req = Request(environ, None)
+        self.assertEqual(repr(req), """<Request "GET '/path?A=B'">""")
+
+    def test_get(self):
+        qs = 'arg1=0&arg2=1&arg1=abc&arg3=def&arg3=1'
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
+        req = Request(environ, None)
+
+        self.assertEqual('0', req.args.get('arg1'))
+        self.assertEqual('def', req.args.get('arg3'))
+
+    def test_getfirst(self):
+        qs = 'arg1=0&arg2=1&arg1=abc&arg3=def&arg3=1'
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
+        req = Request(environ, None)
+
+        self.assertEqual('0', req.args.getfirst('arg1'))
+        self.assertEqual('def', req.args.getfirst('arg3'))
+
+    def test_get_list(self):
+        qs = 'arg1=0&arg2=1&arg1=abc&arg3=def&arg3=1'
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
+        req = Request(environ, None)
+
+        self.assertEqual(['0', 'abc'], req.args.getlist('arg1'))
+        self.assertEqual(['def', '1'], req.args.getlist('arg3'))
+
     def test_as_bool(self):
         qs = 'arg1=0&arg2=1&arg3=yes&arg4=a&arg5=1&arg5=0'
-        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
         req = Request(environ, None)
 
         self.assertIsNone(req.args.as_bool('arg0'))
@@ -113,7 +173,7 @@ class RequestTestCase(unittest.TestCase):
 
     def test_as_int(self):
         qs = 'arg1=1&arg2=a&arg3=3&arg3=4'
-        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
         req = Request(environ, None)
 
         self.assertIsNone(req.args.as_int('arg0'))
@@ -131,7 +191,7 @@ class RequestTestCase(unittest.TestCase):
 
     def test_getbool(self):
         qs = 'arg1=0&arg2=1&arg3=yes&arg4=a&arg5=1&arg5=0'
-        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
         req = Request(environ, None)
 
         self.assertIsNone(req.args.getbool('arg0'))
@@ -140,14 +200,18 @@ class RequestTestCase(unittest.TestCase):
         self.assertFalse(req.args.getbool('arg1', True))
         self.assertTrue(req.args.getbool('arg2'))
         self.assertTrue(req.args.getbool('arg3'))
-        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg4')
-        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg4', True)
-        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg5')
-        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg5', True)
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg4')
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg4', True)
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg5')
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg5', True)
 
     def test_getint(self):
         qs = 'arg1=1&arg2=a&arg3=3&arg3=4'
-        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
         req = Request(environ, None)
 
         self.assertIsNone(req.args.getint('arg0'))
@@ -159,17 +223,102 @@ class RequestTestCase(unittest.TestCase):
         self.assertEqual(0, req.args.getint('arg1', max=0))
         self.assertEqual(0, req.args.getint('arg1', None, max=0))
         self.assertEqual(0, req.args.getint('arg1', None, -1, 0))
-        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg2')
-        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg2', 2)
-        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg3')
-        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg3', 2)
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg2')
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg2', 2)
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg3')
+        with self.assertRaises(HTTPBadRequest):
+            req.args.getbool('arg3', 2)
+
+    def test_getfile(self):
+        file_content = 'The file content.'
+        file_name = 'thefile.txt'
+        form_data = """\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="attachment"; filename="%(file_name)s"\r\n\
+Content-Type: text/plain\r\n\
+\r\n\
+%(file_content)s\r\n\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="action"\r\n\
+\r\n\
+new\r\n\
+--%(boundary)s--\r\n\
+"""
+        boundary = '_BOUNDARY_'
+        content_type = b'multipart/form-data; boundary="%s"' % boundary
+        form_data %= {
+            'boundary': boundary,
+            'file_content': file_content,
+            'file_name': file_name,
+        }
+        environ = _make_environ(method='POST', **{
+            'wsgi.input': io.BytesIO(form_data),
+            'CONTENT_LENGTH': str(len(form_data)),
+            'CONTENT_TYPE': content_type
+        })
+        req = Request(environ, None)
+
+        file_ = req.args.getfile('attachment')
+
+        self.assertEqual(file_name, file_[0])
+        self.assertEqual(file_content, file_[1].getvalue())
+        self.assertEqual(len(file_content), file_[2])
+
+    def test_getfilelist(self):
+        file_content = 'The file0 content.', 'The file1 content.'
+        file_name = 'file0.txt', 'file1.txt'
+        form_data = """\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="attachment"; filename="%(file0_name)s"\r\n\
+Content-Type: text/plain\r\n\
+\r\n\
+%(file0_content)s\r\n\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="attachment"; filename="%(file1_name)s"\r\n\
+Content-Type: text/plain\r\n\
+\r\n\
+%(file1_content)s\r\n\
+--%(boundary)s\r\n\
+Content-Disposition: form-data; name="action"\r\n\
+\r\n\
+new\r\n\
+--%(boundary)s--\r\n\
+"""
+        boundary = '_BOUNDARY_'
+        content_type = b'multipart/form-data; boundary="%s"' % boundary
+        form_data %= {
+            'boundary': boundary,
+            'file0_content': file_content[0],
+            'file0_name': file_name[0],
+            'file1_content': file_content[1],
+            'file1_name': file_name[1],
+        }
+        environ = _make_environ(method='POST', **{
+            'wsgi.input': io.BytesIO(form_data),
+            'CONTENT_LENGTH': str(len(form_data)),
+            'CONTENT_TYPE': content_type
+        })
+        req = Request(environ, None)
+
+        file_ = req.args.getfilelist('attachment')
+
+        self.assertEqual(2, len(file_))
+        self.assertEqual(file_name[0], file_[0][0])
+        self.assertEqual(file_content[0], file_[0][1].getvalue())
+        self.assertEqual(file_name[1], file_[1][0])
+        self.assertEqual(file_content[1], file_[1][1].getvalue())
+        self.assertEqual(len(file_content[1]), file_[1][2])
 
     def test_require(self):
         qs = 'arg1=1'
-        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        environ = _make_environ(method='GET', QUERY_STRING=qs)
         req = Request(environ, None)
 
-        self.assertRaises(HTTPBadRequest, req.args.require, 'arg0')
+        with self.assertRaises(HTTPBadRequest):
+            req.args.require('arg0')
         self.assertIsNone(req.args.require('arg1'))
 
     def test_is_xhr_true(self):
@@ -182,6 +331,24 @@ class RequestTestCase(unittest.TestCase):
         req = Request(environ, None)
         self.assertFalse(req.is_xhr)
 
+    def test_is_authenticated_as_none(self):
+        environ = _make_environ()
+        req = Request(environ, None)
+        req.authname = None
+        self.assertFalse(req.is_authenticated)
+
+    def test_is_authenticated_as_anonymous(self):
+        environ = _make_environ()
+        req = Request(environ, None)
+        req.authname = 'anonymous'
+        self.assertFalse(req.is_authenticated)
+
+    def test_is_authenticated_as_valid_user(self):
+        environ = _make_environ()
+        req = Request(environ, None)
+        req.authname = 'user'
+        self.assertTrue(req.is_authenticated)
+
     def test_base_url(self):
         environ = _make_environ()
         req = Request(environ, None)
@@ -204,7 +371,7 @@ class RequestTestCase(unittest.TestCase):
 
     def test_base_url_https_host(self):
         environ = _make_environ(scheme='https', server_port=443,
-                                     HTTP_HOST='example.com')
+                                HTTP_HOST='example.com')
         req = Request(environ, None)
         self.assertEqual('https://example.com/trac', req.base_url)
 
@@ -215,61 +382,42 @@ class RequestTestCase(unittest.TestCase):
 
     def test_base_url_proxy(self):
         environ = _make_environ(HTTP_HOST='localhost',
-                                     HTTP_X_FORWARDED_HOST='example.com')
+                                HTTP_X_FORWARDED_HOST='example.com')
         req = Request(environ, None)
         self.assertEqual('http://localhost/trac', req.base_url)
 
     def test_languages(self):
-        environ = _make_environ()
-        environ['HTTP_ACCEPT_LANGUAGE'] = 'en-us,en;q=0.5'
+        environ = _make_environ(HTTP_ACCEPT_LANGUAGE='en-us,en;q=0.5')
         req = Request(environ, None)
         self.assertEqual(['en-us', 'en'], req.languages)
 
     def test_redirect(self):
-        status_sent = []
-        headers_sent = {}
-        def start_response(status, headers):
-            status_sent.append(status)
-            headers_sent.update(dict(headers))
-        environ = _make_environ(method='HEAD')
-        req = Request(environ, start_response)
-        req.session = Mock(save=lambda: None)
-        self.assertRaises(RequestDone, req.redirect, '/trac/test')
-        self.assertEqual('302 Found', status_sent[0])
+        req = _make_req(_make_environ(method='HEAD'))
+        with self.assertRaises(RequestDone):
+            req.redirect('/trac/test')
+        self.assertEqual('302 Found', req.status_sent[0])
         self.assertEqual('http://example.org/trac/test',
-                         headers_sent['Location'])
+                         req.headers_sent['Location'])
 
     def test_redirect_absolute(self):
-        status_sent = []
-        headers_sent = {}
-        def start_response(status, headers):
-            status_sent.append(status)
-            headers_sent.update(dict(headers))
-        environ = _make_environ(method='HEAD')
-        req = Request(environ, start_response)
-        req.session = Mock(save=lambda: None)
-        self.assertRaises(RequestDone, req.redirect,
-                          'http://example.com/trac/test')
-        self.assertEqual('302 Found', status_sent[0])
+        req = _make_req(_make_environ(method='HEAD'))
+        with self.assertRaises(RequestDone):
+            req.redirect('http://example.com/trac/test')
+        self.assertEqual('302 Found', req.status_sent[0])
         self.assertEqual('http://example.com/trac/test',
-                         headers_sent['Location'])
+                         req.headers_sent['Location'])
 
     def test_redirect_with_post_and_hash_for_msie(self):
         url = 'http://example.com/trac/ticket/1#comment:2'
         msie303 = 'http://example.com/trac/ticket/1#__msie303:comment:2'
 
         def location(ua):
-            status_sent = []
-            headers_sent = {}
-            def start_response(status, headers):
-                status_sent.append(status)
-                headers_sent.update(dict(headers))
             environ = _make_environ(method='POST', HTTP_USER_AGENT=ua)
-            req = Request(environ, start_response)
-            req.session = Mock(save=lambda: None)
-            self.assertRaises(RequestDone, req.redirect, url)
-            self.assertEqual('303 See Other', status_sent[0])
-            return headers_sent['Location']
+            req = _make_req(environ)
+            with self.assertRaises(RequestDone):
+                req.redirect(url)
+            self.assertEqual('303 See Other', req.status_sent[0])
+            return req.headers_sent['Location']
 
         # IE 11 strict mode
         self.assertEqual(url, location(
@@ -304,58 +452,38 @@ class RequestTestCase(unittest.TestCase):
             'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)'))
 
     def test_write_iterable(self):
-        buf = StringIO()
-        def write(data):
-            buf.write(data)
-        def start_response(status, headers):
-            return write
-        environ = _make_environ(method='GET')
-
-        buf = StringIO()
-        req = Request(environ, start_response)
+        req = _make_req(_make_environ(method='GET'))
         req.send_header('Content-Type', 'text/plain;charset=utf-8')
         req.write(('Foo', 'bar', 'baz'))
-        self.assertEqual('Foobarbaz', buf.getvalue())
+        self.assertEqual('Foobarbaz', req.response_sent)
 
     def test_write_unicode(self):
-        buf = StringIO()
-        def write(data):
-            buf.write(data)
-        def start_response(status, headers):
-            return write
-        environ = _make_environ(method='HEAD')
-
-        req = Request(environ, start_response)
+        req = _make_req(_make_environ(method='HEAD'))
         req.send_header('Content-Type', 'text/plain;charset=utf-8')
         req.send_header('Content-Length', 0)
         # anyway we're not supposed to send unicode, so we get a ValueError
-        self.assertRaises(ValueError, req.write, u'Föö')
-        self.assertRaises(ValueError, req.write, ('F', u'öo'))
+        with self.assertRaises(ValueError):
+            req.write(u'Föö')
+        with self.assertRaises(ValueError):
+            req.write(('F', u'öo'))
 
     def test_send_iterable(self):
-        baton = {'content': StringIO(), 'status': None, 'headers': None}
-        def write(data):
-            baton['content'].write(data)
-        def start_response(status, headers):
-            baton['status'] = status
-            baton['headers'] = headers
-            return write
-        environ = _make_environ(method='GET')
-
         def iterable():
             yield 'line1,'
             yield ''
             yield 'line2,'
             yield 'line3\n'
 
-        req = Request(environ, start_response)
-        self.assertRaises(RequestDone, req.send, iterable())
-        self.assertEqual('200 Ok', baton['status'])
-        self.assertEqual([('Cache-Control', 'must-revalidate'),
-                          ('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT'),
-                          ('Content-Type', 'text/html;charset=utf-8')],
-                         baton['headers'])
-        self.assertEqual('line1,line2,line3\n', baton['content'].getvalue())
+        req = _make_req(_make_environ(method='GET'))
+        with self.assertRaises(RequestDone):
+            req.send(iterable())
+        self.assertEqual('200 Ok', req.status_sent[0])
+        self.assertEqual('must-revalidate', req.headers_sent['Cache-Control'])
+        self.assertEqual('Fri, 01 Jan 1999 00:00:00 GMT',
+                         req.headers_sent['Expires'])
+        self.assertEqual('text/html;charset=utf-8',
+                         req.headers_sent['Content-Type'])
+        self.assertEqual('line1,line2,line3\n', req.response_sent)
 
     def test_invalid_cookies(self):
         environ = _make_environ(HTTP_COOKIE='bad:key=value;')
@@ -369,12 +497,16 @@ class RequestTestCase(unittest.TestCase):
                          str(req.incookie).rstrip(';'))
 
     def test_read(self):
-        environ = _make_environ(**{'wsgi.input': StringIO('test input')})
+        environ = _make_environ(**{
+            'wsgi.input': io.BytesIO(b'test input')
+        })
         req = Request(environ, None)
         self.assertEqual('test input', req.read())
 
     def test_read_size(self):
-        environ = _make_environ(**{'wsgi.input': StringIO('test input')})
+        environ = _make_environ(**{
+            'wsgi.input': io.BytesIO(b'test input')
+        })
         req = Request(environ, None)
         self.assertEqual('test', req.read(size=4))
 
@@ -389,18 +521,16 @@ class RequestTestCase(unittest.TestCase):
             self.fail("HTTPBadRequest not raised.")
 
     def test_qs_with_null_bytes_for_name(self):
-        environ = _make_environ(method='GET',
-                                     **{'QUERY_STRING': 'acti\x00n=fOO'})
+        environ = _make_environ(method='GET', QUERY_STRING='acti\x00n=fOO')
         self._test_qs_with_null_bytes(environ)
 
     def test_qs_with_null_bytes_for_value(self):
-        environ = _make_environ(method='GET',
-                                     **{'QUERY_STRING': 'action=f\x00O'})
+        environ = _make_environ(method='GET', QUERY_STRING='action=f\x00O')
         self._test_qs_with_null_bytes(environ)
 
     def test_post_with_unnamed_value(self):
         boundary = '_BOUNDARY_'
-        form_data = textwrap.dedent("""\
+        form_data = textwrap.dedent(b"""\
             --%(boundary)s\r\n\
             Content-Disposition: form-data; name="foo"\r\n\
             \r\n\
@@ -418,7 +548,7 @@ class RequestTestCase(unittest.TestCase):
         form_data %= {'boundary': boundary}
         content_type = 'multipart/form-data; boundary="%s"' % boundary
         environ = _make_environ(method='POST', **{
-            'wsgi.input': StringIO(form_data),
+            'wsgi.input': io.BytesIO(form_data),
             'CONTENT_LENGTH': str(len(form_data)),
             'CONTENT_TYPE': content_type
         })
@@ -430,11 +560,11 @@ class RequestTestCase(unittest.TestCase):
 
     def _test_post_with_null_bytes(self, form_data):
         boundary = '_BOUNDARY_'
-        content_type = 'multipart/form-data; boundary="%s"' % boundary
+        content_type = b'multipart/form-data; boundary="%s"' % boundary
         form_data %= {'boundary': boundary}
 
         environ = _make_environ(method='POST', **{
-            'wsgi.input': StringIO(form_data),
+            'wsgi.input': io.BytesIO(form_data),
             'CONTENT_LENGTH': str(len(form_data)),
             'CONTENT_TYPE': content_type
         })
@@ -489,53 +619,44 @@ class RequestTestCase(unittest.TestCase):
         """Make sure req.args parsing is consistent even after the backwards
         incompatible change introduced in Python 2.6.
         """
-        environ = _make_environ(method='GET',
-                                     **{'QUERY_STRING': 'action=foo'})
+        environ = _make_environ(method='GET', QUERY_STRING='action=foo')
         req = Request(environ, None)
         self.assertEqual('foo', req.args['action'])
-        environ = _make_environ(method='POST',
-                                     **{'wsgi.input': StringIO('action=bar'),
-                                        'CONTENT_LENGTH': '10',
-                                        'CONTENT_TYPE': 'application/x-www-form-urlencoded',
-                                        'QUERY_STRING': 'action=foo'})
+        environ = _make_environ(method='POST', **{
+            'wsgi.input': io.BytesIO(b'action=bar'),
+            'CONTENT_LENGTH': '10',
+            'CONTENT_TYPE': 'application/x-www-form-urlencoded',
+            'QUERY_STRING': 'action=foo'
+        })
         req = Request(environ, None)
         self.assertEqual('bar', req.args['action'])
 
     def test_qs_invalid_value_bytes(self):
-        environ = _make_environ(**{'QUERY_STRING': 'name=%FF'})
+        environ = _make_environ(QUERY_STRING='name=%FF')
         req = Request(environ, None)
-        self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
+        with self.assertRaises(HTTPBadRequest):
+            req.arg_list()
 
     def test_qs_invalid_name_bytes(self):
-        environ = _make_environ(**{'QUERY_STRING': '%FF=value'})
+        environ = _make_environ(QUERY_STRING='%FF=value')
         req = Request(environ, None)
-        self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
+        with self.assertRaises(HTTPBadRequest):
+            req.arg_list()
 
     def test_post_text_html_disables_xss(self):
         """POST request with content-type text/html disables XSS
         protection (#12926).
         """
-        status_sent = []
-        headers_sent = {}
-        buf = StringIO()
-
-        def write(data):
-            buf.write(data)
-
-        def start_response(status, headers):
-            status_sent.append(status)
-            headers_sent.update(dict(headers))
-            return write
-
         content_type = 'text/html'
         content = "The content"
         environ = _make_environ(method='POST',
-                                **{'wsgi.input': StringIO(content),
+                                **{'wsgi.input': io.BytesIO(content),
                                    'CONTENT_LENGTH': str(len(content)),
                                    'CONTENT_TYPE': content_type})
-        req = Request(environ, start_response)
-        self.assertRaises(RequestDone, req.send, content, content_type)
-        self.assertIn('0', headers_sent['X-XSS-Protection'])
+        req = _make_req(environ)
+        with self.assertRaises(RequestDone):
+            req.send(content, content_type)
+        self.assertIn('0', req.headers_sent['X-XSS-Protection'])
 
     def test_is_valid_header(self):
         # Reserved headers not allowed.
@@ -557,10 +678,7 @@ class RequestTestCase(unittest.TestCase):
 class RequestSendFileTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.status = None
-        self.headers = None
-        self.response = StringIO()
-        self.dir = tempfile.mkdtemp(prefix='trac-')
+        self.dir = mkdtemp()
         self.filename = os.path.join(self.dir, 'test.txt')
         self.data = 'contents\n'
         create_file(self.filename, self.data, 'wb')
@@ -571,176 +689,56 @@ class RequestSendFileTestCase(unittest.TestCase):
             self.req._response.close()
         rmtree(self.dir)
 
-    def _start_response(self, status, headers):
-        self.status = status
-        self.headers = dict(headers)
-        def write(data):
-            self.response.write(data)
-        return write
-
-    def _create_req(self, use_xsendfile=False, xsendfile_header='X-Sendfile',
-                    **kwargs):
-        req = Request(_make_environ(**kwargs), self._start_response)
-        req.callbacks.update({'use_xsendfile': lambda r: use_xsendfile,
-                              'xsendfile_header': lambda r: xsendfile_header})
-        self.req = req
+    def _create_req(self, use_xsendfile=False, xsendfile_header='X-Sendfile'):
+        self.req = req = _make_req(_make_environ(), use_xsendfile=use_xsendfile,
+                                   xsendfile_header=xsendfile_header)
         return req
 
     def test_send_file(self):
         req = self._create_req()
-        self.assertRaises(RequestDone, req.send_file, self.filename,
-                          'text/plain')
-        self.assertEqual('200 Ok', self.status)
-        self.assertEqual('text/plain', self.headers['Content-Type'])
-        self.assertEqual(str(len(self.data)), self.headers['Content-Length'])
-        self.assertNotIn('X-Sendfile', self.headers)
+        with self.assertRaises(RequestDone):
+            req.send_file(self.filename, 'text/plain')
+        self.assertEqual('200 Ok', req.status_sent[0])
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(str(len(self.data)),
+                         req.headers_sent['Content-Length'])
+        self.assertNotIn('X-Sendfile', req.headers_sent)
         self.assertEqual(self.data, ''.join(req._response))
-        self.assertEqual('', self.response.getvalue())
+        self.assertEqual('', req.response_sent)
 
     def test_send_file_with_xsendfile(self):
         req = self._create_req(use_xsendfile=True)
-        self.assertRaises(RequestDone, req.send_file, self.filename,
-                          'text/plain')
-        self.assertEqual('200 Ok', self.status)
-        self.assertEqual('text/plain', self.headers['Content-Type'])
-        self.assertEqual(self.filename, self.headers['X-Sendfile'])
-        self.assertEqual(None, req._response)
-        self.assertEqual('', self.response.getvalue())
+        with self.assertRaises(RequestDone):
+            req.send_file(self.filename, 'text/plain')
+        self.assertEqual('200 Ok', req.status_sent[0])
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(self.filename, req.headers_sent['X-Sendfile'])
+        self.assertIsNone(req._response)
+        self.assertEqual('', req.response_sent)
 
     def test_send_file_with_xsendfile_header(self):
         req = self._create_req(use_xsendfile=True,
                                xsendfile_header='X-Accel-Redirect')
-        self.assertRaises(RequestDone, req.send_file, self.filename,
-                          'text/plain')
-        self.assertEqual('200 Ok', self.status)
-        self.assertEqual('text/plain', self.headers['Content-Type'])
-        self.assertEqual(self.filename, self.headers['X-Accel-Redirect'])
-        self.assertNotIn('X-Sendfile', self.headers)
-        self.assertEqual(None, req._response)
-        self.assertEqual('', self.response.getvalue())
+        with self.assertRaises(RequestDone):
+            req.send_file(self.filename, 'text/plain')
+        self.assertEqual('200 Ok', req.status_sent[0])
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(self.filename, req.headers_sent['X-Accel-Redirect'])
+        self.assertNotIn('X-Sendfile', req.headers_sent)
+        self.assertIsNone(req._response)
+        self.assertEqual('', req.response_sent)
 
     def test_send_file_with_xsendfile_and_empty_header(self):
         req = self._create_req(use_xsendfile=True, xsendfile_header='')
-        self.assertRaises(RequestDone, req.send_file, self.filename,
-                          'text/plain')
-        self.assertEqual('200 Ok', self.status)
-        self.assertEqual('text/plain', self.headers['Content-Type'])
-        self.assertEqual(str(len(self.data)), self.headers['Content-Length'])
-        self.assertNotIn('X-Sendfile', self.headers)
+        with self.assertRaises(RequestDone):
+            req.send_file(self.filename, 'text/plain')
+        self.assertEqual('200 Ok', req.status_sent[0])
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(str(len(self.data)),
+                         req.headers_sent['Content-Length'])
+        self.assertNotIn('X-Sendfile', req.headers_sent)
         self.assertEqual(self.data, ''.join(req._response))
-        self.assertEqual('', self.response.getvalue())
-
-
-class SendErrorTestCase(unittest.TestCase):
-
-    def setUp(self):
-        self.env = EnvironmentStub()
-
-    def tearDown(self):
-        self.env.reset_db()
-
-    def test_trac_error(self):
-        content = self._send_error(error_klass=TracError)
-        self.assertIn('<p class="message">Oops!</p>', content)
-        self.assertNotIn('<strong>Trac detected an internal error:</strong>',
-                         content)
-        self.assertNotIn('There was an internal error in Trac.', content)
-
-    def test_internal_error_for_non_admin(self):
-        content = self._send_error(perm={})
-        self.assertIn('There was an internal error in Trac.', content)
-        self.assertIn('<p>To that end, you could', content)
-        self.assertNotIn('This is probably a local installation issue.',
-                         content)
-        self.assertNotIn('<h2>Found a bug in Trac?</h2>', content)
-
-    def test_internal_error_with_admin_trac_for_non_admin(self):
-        content = self._send_error(perm={},
-                                   admin_trac_url='http://example.org/admin')
-        self.assertIn('There was an internal error in Trac.', content)
-        self.assertIn('<p>To that end, you could', content)
-        self.assertIn(' action="http://example.org/admin/newticket#"', content)
-        self.assertNotIn('This is probably a local installation issue.',
-                         content)
-        self.assertNotIn('<h2>Found a bug in Trac?</h2>', content)
-
-    def test_internal_error_without_admin_trac_for_non_admin(self):
-        content = self._send_error(perm={}, admin_trac_url='')
-        self.assertIn('There was an internal error in Trac.', content)
-        self.assertNotIn('<p>To that end, you could', content)
-        self.assertNotIn('This is probably a local installation issue.',
-                         content)
-        self.assertNotIn('<h2>Found a bug in Trac?</h2>', content)
-
-    def test_internal_error_for_admin(self):
-        content = self._send_error()
-        self.assertNotIn('There was an internal error in Trac.', content)
-        self.assertIn('This is probably a local installation issue.', content)
-        self.assertNotIn('a ticket at the admin Trac to report', content)
-        self.assertIn('<h2>Found a bug in Trac?</h2>', content)
-        self.assertIn('<p>Otherwise, please', content)
-        self.assertIn(' action="http://example.org/tracker/newticket"',
-                      content)
-
-    def test_internal_error_with_admin_trac_for_admin(self):
-        content = self._send_error(admin_trac_url='http://example.org/admin')
-        self.assertNotIn('There was an internal error in Trac.', content)
-        self.assertIn('This is probably a local installation issue.', content)
-        self.assertIn('a ticket at the admin Trac to report', content)
-        self.assertIn(' action="http://example.org/admin/newticket#"', content)
-        self.assertIn('<h2>Found a bug in Trac?</h2>', content)
-        self.assertIn('<p>Otherwise, please', content)
-        self.assertIn(' action="http://example.org/tracker/newticket"',
-                      content)
-
-    def test_internal_error_without_admin_trac_for_admin(self):
-        content = self._send_error(admin_trac_url='')
-        self.assertNotIn('There was an internal error in Trac.', content)
-        self.assertIn('This is probably a local installation issue.', content)
-        self.assertNotIn('a ticket at the admin Trac to report', content)
-        self.assertIn('<h2>Found a bug in Trac?</h2>', content)
-        self.assertIn('<p>Otherwise, please', content)
-        self.assertIn(' action="http://example.org/tracker/newticket"',
-                      content)
-
-    def _send_error(self, admin_trac_url='.', perm=None,
-                    error_klass=ValueError):
-        self.env.config.set('project', 'admin_trac_url', admin_trac_url)
-        self.assertEqual(admin_trac_url, self.env.project_admin_trac_url)
-
-        content = StringIO()
-        result = {'status': None, 'headers': []}
-        def write(data):
-            content.write(data)
-        def start_response(status, headers, exc_info=None):
-            result['status'] = status
-            result['headers'].extend(headers)
-            return write
-        environ = _make_environ()
-        req = _make_req(environ, start_response)
-        try:
-            raise error_klass('Oops!')
-        except:
-            exc_info = sys.exc_info()
-        data = {'title': 'Internal Error',
-                'type': ('internal', 'TracError')[error_klass is TracError],
-                'message': 'Oops!', 'traceback': None, 'frames': [],
-                'shorten_line': shorten_line,
-                'plugins': [], 'faulty_plugins': [],
-                'tracker': 'http://example.org/tracker', 'tracker_args': {},
-                'description': '', 'description_en': '',
-                'get_systeminfo': lambda: ()}
-        if perm is not None:
-            data['perm'] = perm
-
-        self.assertRaises(RequestDone, req.send_error, exc_info, env=self.env,
-                          data=data)
-        content = content.getvalue().decode('utf-8')
-        self.assertIn('<!DOCTYPE ', content)
-        self.assertEqual('500', result['status'].split()[0])
-        self.assertIn(('Content-Type', 'text/html;charset=utf-8'),
-                      result['headers'])
-        return content
+        self.assertEqual('', req.response_sent)
 
 
 class ParseArgListTestCase(unittest.TestCase):
@@ -778,32 +776,36 @@ class HTTPExceptionTestCase(unittest.TestCase):
 
     def test_tracerror_with_string_as_argument(self):
         e1 = TracError('the message')
-        e2 = HTTPInternalError(e1)
+        e2 = HTTPInternalServerError(e1)
         self.assertEqual('500 Trac Error (the message)', unicode(e2))
 
     def test_tracerror_with_fragment_as_argument(self):
         e1 = TracError(tag(tag.b('the message')))
-        e2 = HTTPInternalError(e1)
+        e2 = HTTPInternalServerError(e1)
         self.assertEqual('500 Trac Error (<b>the message</b>)', unicode(e2))
 
     def test_exception_with_string_as_argument(self):
         e1 = Exception('the message')
-        e2 = HTTPInternalError(e1)
+        e2 = HTTPInternalServerError(e1)
         self.assertEqual('500 Internal Server Error (the message)',
                          unicode(e2))
 
     def test_exception_with_fragment_as_argument(self):
         e1 = Exception(tag(tag.b('the message')))
-        e2 = HTTPInternalError(e1)
+        e2 = HTTPInternalServerError(e1)
         self.assertEqual('500 Internal Server Error (<b>the message</b>)',
                          unicode(e2))
 
+    def test_fragment_with_unicode_as_argument(self):
+        e = HTTPInternalServerError(tag.b(u'thé méssägé'))
+        self.assertEqual(u'500 Internal Server Error (<b>thé méssägé</b>)',
+                         unicode(e))
+
 
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(RequestTestCase))
     suite.addTest(unittest.makeSuite(RequestSendFileTestCase))
-    suite.addTest(unittest.makeSuite(SendErrorTestCase))
     suite.addTest(unittest.makeSuite(ParseArgListTestCase))
     suite.addTest(unittest.makeSuite(HTTPExceptionTestCase))
     return suite
