@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -15,14 +15,16 @@ import time
 from datetime import datetime
 import unittest
 
-import trac.tests.compat
-from trac.admin.api import console_date_format
+from trac.admin.api import console_date_format, get_console_locale
+from trac.admin.console import TracAdmin
+from trac.admin.test import TracAdminTestCaseBase
+from trac.core import Component, ComponentMeta, TracError, implements
 from trac.test import EnvironmentStub, MockRequest
-from trac.util.datefmt import format_date, to_datetime
+from trac.util.datefmt import format_date, get_datetime_format_hint, \
+                              time_now, to_datetime
+from trac.web.api import IRequestHandler
 from trac.web.session import DetachedSession, PURGE_AGE, Session, \
                              SessionAdmin, SessionDict, UPDATE_INTERVAL
-from trac.core import TracError
-from trac.util.datefmt import time_now
 
 
 def _prep_session_table(env, spread_visits=False):
@@ -59,6 +61,7 @@ def _prep_session_table(env, spread_visits=False):
     all_list = auth_list + anon_list
     return auth_list, anon_list, all_list
 
+
 def get_session_attrs(env, sid):
     rows = env.db_query("""
         SELECT a.sid, a.name, a.value
@@ -68,10 +71,11 @@ def get_session_attrs(env, sid):
         WHERE s.sid=%s
         """, (sid,))
     if rows:
-        return dict((row[1], row[2]) for row in rows if row[0])
+        return {row[1]: row[2] for row in rows if row[0]}
     else:
         return None
 
+
 def get_session_info(env, sid):
     """
     :since 1.0: changed `db` input parameter to `env`
@@ -101,6 +105,26 @@ class SessionDictTestCase(unittest.TestCase):
 class SessionTestCase(unittest.TestCase):
     """Unit tests for the persistent session support."""
 
+    request_handlers = []
+
+    @classmethod
+    def setUpClass(cls):
+        class DefaultHandlerStub(Component):
+            implements(IRequestHandler)
+
+            def match_request(self, req):
+                pass
+
+            def process_request(req):
+                pass
+
+        cls.request_handlers = [DefaultHandlerStub]
+
+    @classmethod
+    def tearDownClass(cls):
+        for component in cls.request_handlers:
+            ComponentMeta.deregister(component)
+
     def setUp(self):
         self.env = EnvironmentStub()
 
@@ -171,8 +195,8 @@ class SessionTestCase(unittest.TestCase):
         self._test_session_promotion('john@EXAMPLE.LOCAL')  # LDAP username
 
         sessions = self.env.db_query("SELECT sid, authenticated FROM session")
-        self.assertEqual(set([('john', 1), ('j.smith', 1), (u'Jöhn', 1),
-                              ('john@EXAMPLE.LOCAL', 1)]),
+        self.assertEqual({('john', 1), ('j.smith', 1), (u'Jöhn', 1),
+                          ('john@EXAMPLE.LOCAL', 1)},
                          set(sessions))
 
     def _test_new_session_promotion(self, username):
@@ -199,8 +223,8 @@ class SessionTestCase(unittest.TestCase):
         self._test_new_session_promotion('john@EXAMPLE.LOCAL')  # LDAP username
 
         sessions = self.env.db_query("SELECT sid, authenticated FROM session")
-        self.assertEqual(set([('john', 1), ('j.smith', 1), (u'Jöhn', 1),
-                              ('john@EXAMPLE.LOCAL', 1)]),
+        self.assertEqual({('john', 1), ('j.smith', 1), (u'Jöhn', 1),
+                          ('john@EXAMPLE.LOCAL', 1)},
                          set(sessions))
 
     def test_as_int(self):
@@ -220,6 +244,23 @@ class SessionTestCase(unittest.TestCase):
         self.assertEqual(4, session.as_int('baz', 1, min=4))
         self.assertIsNone(session.as_int('bat'))
 
+    def test_as_float(self):
+        with self.env.db_transaction as db:
+            db("INSERT INTO session VALUES ('123456', 1, 0)")
+            db.executemany("""
+                INSERT INTO session_attribute VALUES (%s,%s,%s,%s)
+                """, (('123456', 1, 'foo', 'bar'), ('123456', 1, 'baz', 3.3)))
+
+        req = MockRequest(self.env, authname='123456')
+        session = Session(self.env, req)
+
+        self.assertEqual('bar', session.get('foo'))
+        self.assertEqual(2, session.as_float('foo', 2))
+        self.assertEqual(3.3, session.as_float('baz', 1))
+        self.assertEqual(2.2, session.as_float('baz', 1, max=2.2))
+        self.assertEqual(4.4, session.as_float('baz', 1, min=4.4))
+        self.assertIsNone(session.as_float('bat'))
+
     def test_as_bool(self):
         with self.env.db_transaction as db:
             db("INSERT INTO session VALUES ('123456', 1, 0)")
@@ -447,7 +488,7 @@ class SessionTestCase(unittest.TestCase):
         req = MockRequest(self.env, authname='anonymous')
         req.incookie['trac_session'] = 'john'
         session = Session(self.env, req)
-        self.assertTrue('foo' not in session)
+        self.assertNotIn('foo', session)
         session['foo'] = 'baz'
         session.save()
 
@@ -643,7 +684,7 @@ class SessionTestCase(unittest.TestCase):
         sess_admin = SessionAdmin(self.env)
 
         # Verify the empty case
-        self.assertRaises(StopIteration, sess_admin._get_list([]).next)
+        self.assertRaises(StopIteration, next, sess_admin._get_list([]))
 
         self.assertEqual([i for i in sess_admin._get_list(['authenticated'])],
                          auth_list)
@@ -719,17 +760,17 @@ class SessionTestCase(unittest.TestCase):
                          get_session_attrs(self.env, 'name00'))
         self.assertIn(('name00', 'john', 'john@example.org'),
                       list(self.env.get_known_users()))
-        sess_admin._do_set('default_handler', 'name00', 'SearchModule')
+        sess_admin._do_set('default_handler', 'name00', 'DefaultHandlerStub')
         self.assertEqual({'name': 'john', 'email': 'john@example.org',
-                          'default_handler': 'SearchModule'},
+                          'default_handler': 'DefaultHandlerStub'},
                          get_session_attrs(self.env, 'name00'))
 
         sess_admin._do_set('name', 'name00', '')
         self.assertEqual({'email': 'john@example.org',
-                          'default_handler': 'SearchModule'},
+                          'default_handler': 'DefaultHandlerStub'},
                          get_session_attrs(self.env, 'name00'))
         sess_admin._do_set('email', 'name00', '')
-        self.assertEqual({'default_handler': 'SearchModule'},
+        self.assertEqual({'default_handler': 'DefaultHandlerStub'},
                          get_session_attrs(self.env, 'name00'))
         sess_admin._do_set('default_handler', 'name00', '')
         self.assertEqual({}, get_session_attrs(self.env, 'name00'))
@@ -741,12 +782,12 @@ class SessionTestCase(unittest.TestCase):
         self.assertIn(('name00', 'val00', 'val00'),
                       list(self.env.get_known_users()))
         sess_admin._do_delete('name00')
-        self.assertEqual(None, get_session_attrs(self.env, 'name00'))
+        self.assertIsNone(get_session_attrs(self.env, 'name00'))
         self.assertNotIn(('name00', 'val00', 'val00'),
                          list(self.env.get_known_users()))
 
         sess_admin._do_delete('nothere')
-        self.assertEqual(None, get_session_attrs(self.env, 'nothere'))
+        self.assertIsNone(get_session_attrs(self.env, 'nothere'))
 
         auth_list, anon_list, all_list = _prep_session_table(self.env)
         sess_admin._do_delete('anonymous')
@@ -775,7 +816,7 @@ class SessionTestCase(unittest.TestCase):
             SELECT name, value FROM session_attribute WHERE sid = %s
             """, (anon_list[0][0],))
         self.assertEqual([], rows)
-        self.assertEqual(None, get_session_attrs(self.env, anon_list[0][0]))
+        self.assertIsNone(get_session_attrs(self.env, anon_list[0][0]))
         self.assertEqual({'name': 'val11', 'email': 'val11'},
                          get_session_attrs(self.env, anon_list[1][0]))
 
@@ -812,10 +853,220 @@ class SessionTestCase(unittest.TestCase):
                           u'abc¹₂³xyz')  # Unicode digits
 
 
+class TracAdminTestCase(TracAdminTestCaseBase):
+
+    request_handlers = []
+
+    @classmethod
+    def setUpClass(cls):
+        class DefaultHandlerStub(Component):
+            implements(IRequestHandler)
+
+            def match_request(self, req):
+                pass
+
+            def process_request(req):
+                pass
+
+        cls.request_handlers = [DefaultHandlerStub]
+
+    @classmethod
+    def tearDownClass(cls):
+        for component in cls.request_handlers:
+            ComponentMeta.deregister(component)
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.admin = TracAdmin()
+        self.admin.env_set('', self.env)
+
+    def tearDown(self):
+        self.env = None
+
+    @property
+    def datetime_format_hint(self):
+        return get_datetime_format_hint(get_console_locale(self.env))
+
+    def test_session_list_no_sessions(self):
+        rv, output = self.execute('session list authenticated')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_list_authenticated(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session list authenticated')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_list_anonymous(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session list anonymous')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_list_all(self):
+        _prep_session_table(self.env)
+        if self.admin.interactive:
+            rv, output = self.execute("session list *")
+        else:
+            rv, output = self.execute("session list '*'")
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_list_authenticated_sid(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session list name00')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_list_anonymous_sid(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session list name10:0')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_list_missing_sid(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session list thisdoesntexist')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_add_missing_sid(self):
+        rv, output = self.execute('session add')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_add_duplicate_sid(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session add name00')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_add_sid_all(self):
+        rv, output = self.execute('session add john John john@example.org')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list john')
+        self.assertExpectedResult(output, {
+            'today': format_date(None, console_date_format)
+        })
+
+    def test_session_add_sid(self):
+        rv, output = self.execute('session add john')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list john')
+        self.assertExpectedResult(output, {
+            'today': format_date(None, console_date_format)
+        })
+
+    def test_session_add_sid_name(self):
+        rv, output = self.execute('session add john John')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list john')
+        self.assertExpectedResult(output,  {
+            'today': format_date(None, console_date_format)
+        })
+
+    def test_session_set_attr_name(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session set name name00 JOHN')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list name00')
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_email(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session set email name00 JOHN@EXAMPLE.ORG')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list name00')
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_default_handler(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session set default_handler name00 '
+                                  'DefaultHandlerStub')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list name00')
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_default_handler_invalid(self):
+        _prep_session_table(self.env)
+        rv, output = \
+            self.execute('session set default_handler name00 InvalidModule')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_missing_attr(self):
+        rv, output = self.execute('session set')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_missing_value(self):
+        rv, output = self.execute('session set name john')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_missing_sid(self):
+        rv, output = self.execute('session set name')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_nonexistent_sid(self):
+        rv, output = self.execute('session set name john foo')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_delete_sid(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session delete name00')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list nam00')
+        self.assertExpectedResult(output)
+
+    def test_session_delete_missing_params(self):
+        rv, output = self.execute('session delete')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_session_delete_anonymous(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session delete anonymous')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list *')
+        self.assertExpectedResult(output)
+
+    def test_session_delete_multiple_sids(self):
+        _prep_session_table(self.env)
+        rv, output = self.execute('session delete name00 name01 name02 '
+                                   'name03')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list *')
+        self.assertExpectedResult(output)
+
+    def test_session_purge_age(self):
+        _prep_session_table(self.env, spread_visits=True)
+        rv, output = self.execute('session purge 20100112')
+        self.assertEqual(0, rv, output)
+        rv, output = self.execute('session list *')
+        self.assertExpectedResult(output)
+
+    def test_session_purge_invalid_date(self):
+        rv, output = self.execute('session purge <purge>')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output, {
+            'hint': self.datetime_format_hint,
+            'isohint': get_datetime_format_hint('iso8601')
+        })
+
+    def test_help_session_purge(self):
+        doc = self.get_command_help('session', 'purge')
+        self.assertIn(u'"YYYY-MM-DDThh:mm:ss±hh:mm"', doc)
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(SessionDictTestCase))
     suite.addTest(unittest.makeSuite(SessionTestCase))
+    suite.addTest(unittest.makeSuite(TracAdminTestCase))
     return suite
 
 
