@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2020 Edgewall Software
+# Copyright (C) 2012-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,15 +12,14 @@
 # history and logs, available at https://trac.edgewall.org/log/.
 
 import os
-import tempfile
 import textwrap
 import unittest
 
-import trac.tests.compat
+from trac.core import Component, ComponentMeta, implements
 from trac.config import ConfigurationError
-from trac.perm import PermissionCache
+from trac.perm import IPermissionRequestor, PermissionCache
 from trac.resource import Resource
-from trac.test import EnvironmentStub, Mock
+from trac.test import EnvironmentStub, Mock, mkdtemp
 from trac.util import create_file
 from trac.versioncontrol.api import Repository
 from tracopt.perm.authz_policy import AuthzPolicy
@@ -28,9 +27,26 @@ from tracopt.perm.authz_policy import AuthzPolicy
 
 class AuthzPolicyTestCase(unittest.TestCase):
 
+    @classmethod
+    def setUpClass(cls):
+        class TestPermissionRequestor(Component):
+            implements(IPermissionRequestor)
+
+            def get_permission_actions(self):
+                perms = ['TEST_VIEW', 'TEST_CREATE', 'TEST_DELETE',
+                         'TEST_MODIFY']
+                return [('TEST_ADMIN', perms)] + perms
+
+        cls.permission_requestors = [TestPermissionRequestor]
+
+    @classmethod
+    def tearDownClass(cls):
+        for component in cls.permission_requestors:
+            ComponentMeta.deregister(component)
+
     def setUp(self):
-        tmpdir = tempfile.mkdtemp(prefix='trac-')
-        self.authz_file = os.path.join(tmpdir, 'trac-authz-policy')
+        temp_dir = mkdtemp()
+        self.authz_file = os.path.join(temp_dir, 'trac-authz-policy')
         create_file(self.authz_file, textwrap.dedent("""\
             # -*- coding: utf-8 -*-
             # Unicode user names
@@ -75,8 +91,10 @@ class AuthzPolicyTestCase(unittest.TestCase):
             [milestone:milestone1]
             anonymous = MILESTONE_VIEW
             """))
-        self.env = EnvironmentStub(enable=['trac.*', AuthzPolicy],
-                                   path=tmpdir)
+        self.env = EnvironmentStub(enable=['trac.perm.*', AuthzPolicy] +
+                                          self.permission_requestors,
+                                   path=temp_dir,
+                                   config=[('logging', 'log_level', 'WARNING')])
         self.env.config.set('trac', 'permission_policies',
                             'AuthzPolicy, DefaultPermissionPolicy')
         self.env.config.set('authz_policy', 'authz_file', self.authz_file)
@@ -191,11 +209,38 @@ class AuthzPolicyTestCase(unittest.TestCase):
         """Metagroup authenticated inherits all permissions granted to
         anonymous.
         """
-        resource  = Resource('milestone', 'milestone1')
+        resource = Resource('milestone', 'milestone1')
         self.assertTrue(self.check_permission('MILESTONE_VIEW',
                                               'anonymous', resource))
         self.assertTrue(self.check_permission('MILESTONE_VIEW',
                                               'authenticated', resource))
+        self.assertIn('MILESTONE_VIEW', self.get_perm('anonymous',
+                                                      resource))
+        self.assertIn('MILESTONE_VIEW', self.get_perm('authenticated',
+                                                      resource))
+
+    def test_undefined_action_is_logged(self):
+        """Undefined action is logged at warning level."""
+        create_file(self.authz_file, textwrap.dedent("""\
+            [groups]
+            administrators = éat
+            [wiki:WikiStart]
+            änon = UNKNOWN_VIEW, TEST_CREATE, !TEST_MODIFY
+            [milestone:milestone1]
+            * = UNKNOWN_MODIFY, !TEST_VIEW
+            """))
+        authz_policy = AuthzPolicy(self.env)
+        authz_policy.parse_authz()
+
+        self.assertEqual(2, len(self.env.log_messages))
+        self.assertIn(('WARNING',
+                       'The action UNKNOWN_VIEW in the [wiki:WikiStart] '
+                       'section of trac-authz-policy is not a valid action.'),
+                      self.env.log_messages)
+        self.assertIn(('WARNING',
+                       'The action UNKNOWN_MODIFY in the [milestone:milestone1] '
+                       'section of trac-authz-policy is not a valid action.'),
+                      self.env.log_messages)
 
     def test_get_authz_file(self):
         """get_authz_file should resolve a relative path."""
